/***************************************************************************
 *   Copyright (C) 2007 by Alexandru Scvortov   *
 *   scvalex@gmail.com   *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <gtk/gtk.h>

#include <iostream>
#include <vector>
#include <cstdio>
#include <cstdlib>

const int MAX_ROW = 10;
const int NUM_FIELDS = 4; // This can't be larger than 10.
const int MAX_ROUNDS = 10;

enum EndCondition {
    FAILITURE,
    SUCCES,
    MISTAKE
};

enum GameType {
    JustGuess,
    Alternating
};

enum Guesser {
    Player,
    Computer
};

GtkWidget *window;

// The main widget.
GtkWidget *nb; // Notebook

// Game
GtkWidget *gamePlayBox; // VButtonBox
GtkWidget *roundL; // Label
GtkWidget *gameDoneB; // Button

// Options
GtkWidget *optGTSimpleRB; // RadioButton
GtkWidget *optGTAlternatingRB; // RadioButton

// End game
GtkWidget *egStatusL; // Label
GtkWidget *egSummaryL; // Label

// The following for widgets used to be declared as pointers
GtkWidget *entryB[NUM_FIELDS]; // Entry
GtkWidget *resultL[2]; // Label

GtkWidget *entryL[NUM_FIELDS]; // Label
GtkWidget *resultB[2]; // Entry

void newGameClicked();
void optionsClicked();
void quitNow();
void mainMenuClicked();
void doneClicked();

bool addNewGameRowDigits();
bool addNewGameRowResults();
int curRow;

int curRound;
void newRound();

int numGuesses;

void showEndGame(EndCondition c);

void generateSecretCode();
short sCode[NUM_FIELDS];

bool checkAttempt();

GameType gt;
void gameTypeChanged();
Guesser g;

int cnc(short a[NUM_FIELDS], short b[NUM_FIELDS]);

std::vector< std::vector<short> > pos;
void initPos();
bool np(std::vector<short> &);
void elimPos();

void newGameClicked() {
    std::cerr << "New Game" << std::endl;

    curRound = 0;
    numGuesses = 0;

    newRound();
}

void removeChild(GtkWidget *child, gpointer parent) {
    gtk_container_remove(GTK_CONTAINER(parent), child);
}

void newRound() {
    ++curRound;
    if (curRound > MAX_ROUNDS)
        return showEndGame(SUCCES);

    char tmp[64];
    sprintf(tmp, "<span font_desc=\"12\">Round: %d</span>", curRound);
    gtk_label_set_markup(GTK_LABEL(roundL), tmp);

    // Removal of old elements
    gtk_container_foreach(GTK_CONTAINER(gamePlayBox), removeChild, gamePlayBox);

    // Restart the counter
    curRow = 0;

    if ((gt == JustGuess) || ((gt == Alternating)) && (curRound % 2 == 1)) {
        std::cerr << "For the next round, player will guess" << std::endl;
        g = Player;

        // Other initialisations
        for (int i(0); i < NUM_FIELDS; ++i)
            entryB[i] = 0;

        resultL[0] = 0; resultL[1] = 0;

        generateSecretCode();

        addNewGameRowDigits();
    } else if ((gt == Alternating) && (curRound % 2 == 0)) {
        std::cerr << "For the next round, computer will guess" << std::endl;
        g = Computer;

        for (int i(0); i < NUM_FIELDS; ++i)
            entryL[i] = 0;

        resultB[0] = 0; resultB[1] = 0;

        initPos();

        addNewGameRowResults();
    }

    gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 1);
}

void optionsClicked() {
    std::cerr << "Options" << std::endl;
    gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 2);
}

void quitNow() {
    std::cerr << "Quit" << std::endl;
    gtk_main_quit();
}

void mainMenuClicked() {
    std::cerr << "Mainmenu" << std::endl;
    gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 0);
}


void helpClicked() {
    std::cerr << "Help" << std::endl;
    gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 3);
}

void doneClicked() {
    std::cerr << "Done" << std::endl;

    if (g == Player) {
        ++numGuesses;

        if (checkAttempt())
            return newRound();

        if (!addNewGameRowDigits())
            return showEndGame(FAILITURE);
    } else {
        if (atoi(gtk_entry_get_text(GTK_ENTRY(resultB[0]))) == 4)
            return newRound();

        elimPos();

        if (pos.size() < 1) {
            std::cerr << "User is an idiot" << std::endl;
            return showEndGame(MISTAKE);
        }

        if (!addNewGameRowResults())
            return showEndGame(SUCCES);
    }
}

int incEntryAndCheckResult(GtkWidget *entry, gpointer nothing) {
    std::cerr << "Activate!" << std::endl;

    int value = 1 + atoi(gtk_entry_get_text(GTK_ENTRY(entry)));

    if (value >= 5)
        value = 0;

    char tmp[10];
    sprintf(tmp, "%d", value);
    gtk_entry_set_text(GTK_ENTRY(entry), tmp);

    int total = atoi(gtk_entry_get_text(GTK_ENTRY(resultB[0]))) + 
                atoi(gtk_entry_get_text(GTK_ENTRY(resultB[1])));

    if (total > 4)
        gtk_widget_set_sensitive(GTK_WIDGET(gameDoneB), FALSE);
    else
        gtk_widget_set_sensitive(GTK_WIDGET(gameDoneB), TRUE);

    return TRUE;
}

//! Adds a new game row on which the player can change the results.
bool addNewGameRowResults() {
    if (curRow >= MAX_ROW)
        return false;

    ++curRow;

    static GtkWidget *pBox = 0; // HButtonBox

    if (curRow > 1) {
        std::string aux1 = gtk_entry_get_text(GTK_ENTRY(resultB[0])) + std::string("C");
        std::string aux2 = gtk_entry_get_text(GTK_ENTRY(resultB[1])) + std::string("NC");
        gtk_container_remove(GTK_CONTAINER(pBox), resultB[0]);
        gtk_container_remove(GTK_CONTAINER(pBox), resultB[1]);

        GtkWidget *l = gtk_label_new(aux1.c_str()); // Label
        gtk_box_pack_start_defaults(GTK_BOX(pBox), l);
        gtk_widget_show(l);
        l = gtk_label_new(aux2.c_str());
        gtk_box_pack_start_defaults(GTK_BOX(pBox), l);
        gtk_widget_show(l);
    }

    GtkWidget *box = gtk_hbox_new(false, 10); // HBox
    pBox = box;

    gtk_box_pack_start_defaults(GTK_BOX(gamePlayBox), box);
    gtk_widget_show(box);

    char txt[4];
    sprintf(txt, "%d.", curRow);
    GtkWidget *l = gtk_label_new(txt); // Label
    gtk_box_pack_start_defaults(GTK_BOX(box), l);
    gtk_widget_show(l);

    for (int i(0); i < NUM_FIELDS; ++i) {
        char tmp[40];
        sprintf(tmp, "<span weight=\"bold\">%d</span>", pos[0][i]);
        std::cerr << tmp;
        entryL[i] = gtk_label_new("");
        gtk_label_set_markup(GTK_LABEL(entryL[i]), tmp);
        gtk_box_pack_start(GTK_BOX(pBox), entryL[i], FALSE, FALSE, 20);
        gtk_widget_show(entryL[i]);
    }
    std::cerr << std::endl;

    for (int i(0); i < 2; ++i) {
        resultB[i] = gtk_entry_new();
        gtk_entry_set_text(GTK_ENTRY(resultB[i]), "0");
        gtk_entry_set_width_chars(GTK_ENTRY(resultB[i]), 2);
        g_signal_connect(GTK_ENTRY(resultB[i]), "button-press-event", G_CALLBACK(incEntryAndCheckResult), 0);
        gtk_box_pack_start_defaults(GTK_BOX(box), resultB[i]);
        gtk_widget_show(resultB[i]);
    }
    gtk_widget_set_sensitive(GTK_WIDGET(gameDoneB), TRUE);

    return true;
}

int incEntryAndCheckCode(GtkWidget *entry, gpointer nothing) {
    std::cerr << "Activate!" << std::endl;

    int value = 1 + atoi(gtk_entry_get_text(GTK_ENTRY(entry)));

    if (value >= 10)
        value = 0;

    char tmp[10];
    sprintf(tmp, "%d", value);
    gtk_entry_set_text(GTK_ENTRY(entry), tmp);

    std::vector<int> na(10, 0);
    bool repeats(false);
    for (int i(0); i < NUM_FIELDS; ++i) {
        int aux = ++na[atoi(gtk_entry_get_text(GTK_ENTRY(entryB[i])))];
        if (aux > 1)
            repeats = true;
    }

    if (repeats)
        gtk_widget_set_sensitive(GTK_WIDGET(gameDoneB), FALSE);
    else
        gtk_widget_set_sensitive(GTK_WIDGET(gameDoneB), TRUE);

    return TRUE;
}

//! Adds a new game row on which the player can change the digits.
bool addNewGameRowDigits() {
    if (curRow >= MAX_ROW)
        return false;

    ++curRow;

    static GtkWidget *pBox = 0; // HButtonBox

    if (curRow > 1) {
        std::string aux1 = gtk_label_get_text(GTK_LABEL(resultL[0]));
        std::string aux2 = gtk_label_get_text(GTK_LABEL(resultL[1]));
        std::string aux[NUM_FIELDS];
        gtk_container_remove(GTK_CONTAINER(pBox), resultL[0]);
        gtk_container_remove(GTK_CONTAINER(pBox), resultL[1]);

        for (int i(0); i < NUM_FIELDS; ++i) {
            aux[i] = gtk_entry_get_text(GTK_ENTRY(entryB[i]));
            gtk_container_remove(GTK_CONTAINER(pBox), entryB[i]);
        }

        for (int i(0); i < NUM_FIELDS; ++i) {
            GtkWidget *l = gtk_label_new(""); // Label
            gtk_label_set_markup(GTK_LABEL(l), ("<span weight=\"bold\">" + aux[i] + "</span>").c_str());
            gtk_box_pack_start_defaults(GTK_BOX(pBox), l);
            gtk_widget_show(l);
        }
        GtkWidget *l = gtk_label_new(aux1.c_str()); // Label
        gtk_box_pack_start_defaults(GTK_BOX(pBox), l);
        gtk_widget_show(l);

        l = gtk_label_new(aux2.c_str());
        gtk_box_pack_start_defaults(GTK_BOX(pBox), l);
        gtk_widget_show(l);
    }

    GtkWidget *box = gtk_hbox_new(FALSE, 10); // HButtonBox
    pBox = box;

    gtk_box_pack_start_defaults(GTK_BOX(gamePlayBox), box);
    gtk_widget_show(box);

    char txt[4];
    sprintf(txt, "%d.", curRow);
    GtkWidget *l = gtk_label_new(txt); // Label
    gtk_box_pack_start_defaults(GTK_BOX(box), l);
    gtk_widget_show(l);

    for (int i(0); i < NUM_FIELDS; ++i) {
        entryB[i] = gtk_entry_new();
        gtk_entry_set_text(GTK_ENTRY(entryB[i]), "0");
        gtk_entry_set_width_chars(GTK_ENTRY(entryB[i]), 2);
        g_signal_connect(GTK_ENTRY(entryB[i]), "button-press-event", G_CALLBACK(incEntryAndCheckCode), 0);
        gtk_box_pack_start_defaults(GTK_BOX(box), entryB[i]);
        gtk_widget_show(entryB[i]);
    }
    gtk_widget_set_sensitive(GTK_WIDGET(gameDoneB), FALSE);

    for (int i(0); i < 2; ++i) {
        resultL[i] = gtk_label_new("");
        gtk_box_pack_start_defaults(GTK_BOX(box), resultL[i]);
        gtk_widget_show(resultL[i]);
    }

    gtk_label_set_text(GTK_LABEL(resultL[0]), "0C");
    gtk_label_set_text(GTK_LABEL(resultL[1]), "0NC");

    return true;
}

void showEndGame(EndCondition c) {
    std::cerr << "EndGame" << std::endl;

    if (c == FAILITURE)
        gtk_label_set_markup(GTK_LABEL(egStatusL), "<span font_desc=\"12\" foreground=\"red\">You lose!</span>");
    else if (c == SUCCES)
        gtk_label_set_markup(GTK_LABEL(egStatusL), "<span font_desc=\"12\" foreground=\"green\">You win!</span>");
    else if (c = MISTAKE)
        gtk_label_set_markup(GTK_LABEL(egStatusL), "<span font_desc=\"12\" foreground=\"yellow\">Wrong!</span>");

    char tmp[128];
    sprintf(tmp, "<span font_desc=\"10\">You lasted %d round(s)</span>\n"
                 "<span font_desc=\"10\">You made %d guesses</span>", curRound - 1, numGuesses);
    gtk_label_set_markup(GTK_LABEL(egSummaryL), tmp);

    gtk_notebook_set_current_page(GTK_NOTEBOOK(nb), 4);
}

void generateSecretCode() {
    std::cerr << "Generating secret code" << std::endl;

    short d[10];
    for (int i(0); i < 10; ++i)
        d[i] = i;

    for (int i(0); i < 10; ++i) {
        int ind = (int)(9.0 * (rand() / (RAND_MAX + 1.0)));
        int aux = d[i];
        d[i] = d[ind];
        d[ind] = aux;
    }

    if (d[0] == 0) {
        d[0] = d[1];
        d[1] = 0;
    }

    for (int i(0); i < NUM_FIELDS; ++i) {
        sCode[i] = d[i];
        std::cerr << sCode[i];
    }
    std::cerr << std::endl;
}

bool checkAttempt() {
    std::cerr << "Checking attempt" << std::endl;

    short at[NUM_FIELDS];
    for (int i(0); i < NUM_FIELDS; ++i)
        at[i] = atoi(gtk_entry_get_text(GTK_ENTRY(entryB[i])));

    char tmp[4];

    int r = cnc(at, sCode);
    int c = r / 10; // The number of centered digits in the attempt.
    int nc = r % 10; // The number of noncentered digits.

    sprintf(tmp, "%dC", c);
    gtk_label_set_text(GTK_LABEL(resultL[0]), tmp);

    if (c == NUM_FIELDS)
        return true;

    sprintf(tmp, "%dNC", nc);
    gtk_label_set_text(GTK_LABEL(resultL[1]), tmp);

    return false;
}

void gameTypeChanged() {
    if (gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(optGTSimpleRB)))
        gt = JustGuess;
    else
        gt = Alternating;

    std::cerr << "Game type is now " << ((gt == JustGuess) ? ("Just Guess") : ("Alternating")) << std::endl;
}

int cnc(short aa[NUM_FIELDS], short bb[NUM_FIELDS]) {
    short a[NUM_FIELDS];
    for (int i(0); i < NUM_FIELDS; ++i)
        a[i] = aa[i];

    short b[NUM_FIELDS];
    for (int i(0); i < NUM_FIELDS; ++i)
        b[i] = bb[i];

    int c(0); // The number of centered digits in the attempt.
    for (int i(0); i < NUM_FIELDS; ++i)
        if (a[i] == b[i]) {
            ++c;
            a[i] = -1; // I'm no longet intrested in this digit.
        }

    int nc(0); // The number of noncentered digits.
    for (int i(0); i < NUM_FIELDS; ++i)
        for (int j(0); j < NUM_FIELDS; ++j)
            if (b[i] == a[j]) {
                ++nc;
                a[j] = -1; // I'm no longet intrested in this digit.
                break;
            }

    return c * 10 + nc;
}

bool np(std::vector<short> &v) {
    bool right(false);

    while (!right) {
        int i(NUM_FIELDS - 1);
        ++v[i];
        while ((i >= 0) && (v[i] > 9)) {
            v[i] = 0;
            --i;

            if (i >= 0)
                ++v[i];
        }

        if (i < 0)
            return false;

        right = true;
        for (i = 0; i < NUM_FIELDS; ++i)
            for (int j = i + 1; j < NUM_FIELDS; ++j)
                if (v[i] == v[j])
                    right = false;
    }

    return true;
}

void initPos() {
    std::cerr << "Initialising posibilities" << std::endl;

    std::vector<short> aux;
    for (int i(0); i < NUM_FIELDS; ++i)
        aux.push_back(i);
    --aux[NUM_FIELDS - 1];

    while (np(aux)) {
        pos.push_back(std::vector<short>(aux));
//         for (int i(0); i < NUM_FIELDS; ++i)
//             std::cerr << aux[i];
//         std::cerr << std::endl;
    }
}

void elimPos() {
    int res = atoi(gtk_entry_get_text(GTK_ENTRY(resultB[0]))) * 10 +
              atoi(gtk_entry_get_text(GTK_ENTRY(resultB[1])));

    short auxx[NUM_FIELDS];
    for (int j(0); j < NUM_FIELDS; ++j)
        auxx[j] = pos[0][j];

    std::vector< std::vector<short> > n;

    std::vector< std::vector<short> >::iterator i = pos.begin();
    while (i != pos.end()) {
        short aux[NUM_FIELDS];
        for (int j(0); j < NUM_FIELDS; ++j)
            aux[j] = (*i)[j];

        if (cnc(aux, auxx) == res)
            n.push_back(std::vector<short>(*i));

        ++i;
    }

    pos = n;

    std::cerr << "There are now only " << pos.size() << " valid possibilities" << std::endl;
}

void mainWindow() {
    curRow = 0;

    nb = gtk_notebook_new();
    gtk_container_add(GTK_CONTAINER(window), nb);
    gtk_notebook_set_show_tabs(GTK_NOTEBOOK(nb), FALSE);
    gtk_widget_show(nb);

    // MainMenu
    GtkWidget *topBox; // VButtonBox
    GtkWidget *titleL; // Label
    GtkWidget *butBox; // VButtonBox
    GtkWidget *newGameB; // Button
    GtkWidget *optionsB; // Button
    GtkWidget *helpB; // Button
    GtkWidget *quitB; // Button

    topBox = gtk_vbutton_box_new();
    gtk_widget_show(topBox);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb), topBox, gtk_label_new("MainMenu"));

    gtk_button_box_set_layout(GTK_BUTTON_BOX(topBox), GTK_BUTTONBOX_START);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(topBox), 40);
    gtk_container_set_border_width(GTK_CONTAINER(topBox), 40);

    titleL = gtk_label_new("");
    gtk_label_set_markup(GTK_LABEL(titleL), "<span font_desc=\"24\">C/Nc</span>");
    gtk_box_pack_start(GTK_BOX(topBox), titleL, FALSE, FALSE, 0);
    gtk_widget_show(titleL);

    butBox = gtk_vbutton_box_new();
    gtk_box_pack_start(GTK_BOX(topBox), butBox, FALSE, FALSE, 0);
    gtk_widget_show(butBox);

    gtk_button_box_set_layout(GTK_BUTTON_BOX(butBox), GTK_BUTTONBOX_SPREAD);
    newGameB = gtk_button_new_with_label("New Game");
    gtk_box_pack_start(GTK_BOX(butBox), newGameB, TRUE, FALSE, 0);
    g_signal_connect(GTK_BUTTON(newGameB), "clicked", G_CALLBACK(newGameClicked), 0);
    gtk_widget_show(newGameB);

    optionsB = gtk_button_new_with_label("Options");
    gtk_box_pack_start(GTK_BOX(butBox), optionsB, TRUE, FALSE, 0);
    g_signal_connect(GTK_BUTTON(optionsB), "clicked", G_CALLBACK(optionsClicked), 0);
    gtk_widget_show(optionsB);

    helpB = gtk_button_new_with_label("Help");
    gtk_box_pack_start(GTK_BOX(butBox), helpB, TRUE, FALSE, 0);
    g_signal_connect(GTK_BUTTON(helpB), "clicked", G_CALLBACK(helpClicked), 0);
    gtk_widget_show(helpB);

    quitB = gtk_button_new_with_label("Quit");
    gtk_box_pack_start(GTK_BOX(butBox), quitB, TRUE, FALSE, 0);
    g_signal_connect(GTK_BUTTON(quitB), "clicked", G_CALLBACK(quitNow), 0);
    gtk_widget_show(quitB);

    // Game
    GtkWidget *gameBox; // VBox
    GtkWidget *gameMenuBox; // HButtonBox
    GtkWidget *gameMainMenuB; // Button

    gameBox = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb), gameBox, gtk_label_new("Game"));
    gtk_box_set_homogeneous(GTK_BOX(gameBox), FALSE);
    gtk_box_set_spacing(GTK_BOX(gameBox), 10);
    gtk_container_set_border_width(GTK_CONTAINER(gameBox), 10);
    gtk_widget_show(gameBox);

    gameMenuBox = gtk_hbutton_box_new();
    gtk_box_pack_start(GTK_BOX(gameBox), gameMenuBox, FALSE, FALSE, 0);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(gameMenuBox), GTK_BUTTONBOX_EDGE);
    gtk_widget_show(gameMenuBox);

    gameMainMenuB = gtk_button_new_with_label("Main menu");
    gtk_box_pack_start_defaults(GTK_BOX(gameMenuBox), gameMainMenuB);
    g_signal_connect(GTK_BUTTON(gameMainMenuB), "clicked", G_CALLBACK(mainMenuClicked), 0);
    gtk_widget_show(gameMainMenuB);

    gameDoneB = gtk_button_new_with_label("Done");
    gtk_box_pack_start_defaults(GTK_BOX(gameMenuBox), gameDoneB);
    g_signal_connect(GTK_BUTTON(gameDoneB), "clicked", G_CALLBACK(doneClicked), 0);
    gtk_widget_show(gameDoneB);

    roundL = gtk_label_new("");
    gtk_box_pack_start(GTK_BOX(gameBox), roundL, FALSE, FALSE, 0);
    gtk_widget_show(roundL);

    gamePlayBox = gtk_vbutton_box_new();
    gtk_box_pack_start(GTK_BOX(gameBox), gamePlayBox, FALSE, FALSE, 0);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(gamePlayBox), GTK_BUTTONBOX_START);
    gtk_widget_show(gamePlayBox);

    // Options
    GtkWidget *optBox; // VBox
    GtkWidget *optMenuBox; // HButtonBox
    GtkWidget *optMainMenuB; // Button
    GtkWidget *optGameTypeF; // Frame
    GtkWidget *optGameTypeBox; // VButtonBox

    optBox = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb), optBox, gtk_label_new("Options"));
    gtk_box_set_spacing(GTK_BOX(optBox), 10);
    gtk_container_set_border_width(GTK_CONTAINER(optBox), 10);
    gtk_widget_show(optBox);

    optMenuBox = gtk_hbutton_box_new();
    gtk_box_pack_start(GTK_BOX(optBox), optMenuBox, FALSE, FALSE, 0);
    gtk_widget_show(optMenuBox);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(optMenuBox), GTK_BUTTONBOX_START);

    optMainMenuB = gtk_button_new_with_label("Main menu");
    gtk_box_pack_start_defaults(GTK_BOX(optMenuBox), optMainMenuB);
    g_signal_connect(GTK_BUTTON(optMainMenuB), "clicked", G_CALLBACK(mainMenuClicked), 0);
    gtk_widget_show(optMainMenuB);

    optGameTypeF = gtk_frame_new("Game type");
    gtk_box_pack_start(GTK_BOX(optBox), optGameTypeF, FALSE, FALSE, 0);
    gtk_widget_show(optGameTypeF);

    optGameTypeBox = gtk_vbutton_box_new();
    gtk_container_add(GTK_CONTAINER(optGameTypeF), optGameTypeBox);
    gtk_widget_show(optGameTypeBox);

    optGTSimpleRB = gtk_radio_button_new_with_label(0, "Simple");
    optGTAlternatingRB = gtk_radio_button_new_with_label(gtk_radio_button_get_group(GTK_RADIO_BUTTON(optGTSimpleRB)), "Alternating");
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(optGTSimpleRB), TRUE);
    gt = JustGuess;

    gtk_box_pack_start_defaults(GTK_BOX(optGameTypeBox), optGTSimpleRB);
    g_signal_connect(GTK_BUTTON(optGTSimpleRB), "clicked", G_CALLBACK(gameTypeChanged), 0);
    gtk_widget_show(optGTSimpleRB);

    gtk_box_pack_start_defaults(GTK_BOX(optGameTypeBox), optGTAlternatingRB);
    g_signal_connect(GTK_BUTTON(optGTAlternatingRB), "clicked", G_CALLBACK(gameTypeChanged), 0);
    gtk_widget_show(optGTAlternatingRB);

    // Help
    GtkWidget *helpBox; // VBox
    GtkWidget *helpMenuBox; // HButtonBox
    GtkWidget *helpMainMenuB; // Button
    GtkWidget *helpNB; // Notebook
    GtkWidget *rulesSW; // ScrolledWindow
    GtkWidget *rulesBox; // VBox
    GtkWidget *rulesL; // Label
    GtkWidget *aboutBox; // VBox
    GtkWidget *copyL; // Label

    helpBox = gtk_vbox_new(FALSE, 0);
    gtk_notebook_append_page(GTK_NOTEBOOK(nb), helpBox, gtk_label_new("Help"));
    gtk_container_set_border_width(GTK_CONTAINER(helpBox), 10);
    gtk_widget_show(helpBox);

    helpMenuBox = gtk_hbutton_box_new();
    gtk_box_pack_start(GTK_BOX(helpBox), helpMenuBox, FALSE, FALSE, 10);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(helpMenuBox), GTK_BUTTONBOX_START);
    gtk_widget_show(helpMenuBox);

    helpMainMenuB = gtk_button_new_with_label("Main menu");
    gtk_box_pack_start_defaults(GTK_BOX(helpMenuBox), helpMainMenuB);
    g_signal_connect(GTK_BUTTON(helpMainMenuB), "clicked", G_CALLBACK(mainMenuClicked), 0);
    gtk_widget_show(helpMainMenuB);

    helpNB = gtk_notebook_new();
    gtk_box_pack_start(GTK_BOX(helpBox), helpNB, TRUE, TRUE, 0);
    gtk_notebook_set_tab_pos(GTK_NOTEBOOK(helpNB), GTK_POS_BOTTOM);
    gtk_widget_show(helpNB);

    rulesSW = gtk_scrolled_window_new(0, 0);
    gtk_scrolled_window_set_policy(GTK_SCROLLED_WINDOW(rulesSW), GTK_POLICY_NEVER, GTK_POLICY_AUTOMATIC);
    gtk_notebook_append_page(GTK_NOTEBOOK(helpNB), rulesSW, gtk_label_new("Rules"));
    gtk_widget_show(rulesSW);

    rulesBox = gtk_vbox_new(FALSE, 0);
    gtk_scrolled_window_add_with_viewport(GTK_SCROLLED_WINDOW(rulesSW), rulesBox);
    gtk_widget_show(rulesBox);

    rulesL = gtk_label_new("");
    gtk_label_set_text(GTK_LABEL(rulesL),
        "This is a Mastermind (Enigma, Guess, etc.) style GTK+ game. It implements two"
        " modes of play: player guesses computer's code and computer guesses player's"
        " code. The two modes alternate between rounds.\n"
        "\n"
        "A code is a four digit number in which no two digits are the same. In"
        " addition, the number cannot start with 0. After the player inputs his guess"
        " the computer returns two number: the number of centered digits and the number"
        " of non-centered digits.\n"
        "\n"
        "A centered digit is a digit that appears in the code and is in the correct"
        " postition. Ex. code: 1234 guess:1537; there are two centered digit: 1 and 3.\n"
        "\n"
        "A non-centered digit is a digit that appears in the code but is NOT in the"
        " correct position. Ex. code: 1234 guess: 4281; there are two non-centered"
        " digits: 4 and 1.\n"
        "\n"
        "The goal is to guess the code."
    );
    gtk_label_set_line_wrap(GTK_LABEL(rulesL), TRUE);
    gtk_box_pack_start(GTK_BOX(rulesBox), rulesL, TRUE, TRUE, 0);
    gtk_widget_show(rulesL);

    aboutBox = gtk_vbutton_box_new();
    gtk_button_box_set_layout(GTK_BUTTON_BOX(aboutBox), GTK_BUTTONBOX_SPREAD);
    gtk_notebook_append_page(GTK_NOTEBOOK(helpNB), aboutBox, gtk_label_new("About"));
    gtk_widget_show(aboutBox);

    copyL = gtk_label_new("");
    gtk_label_set_text(GTK_LABEL(copyL),
        "C-NC - A Mastermind style game for handhelds\n"
        "\n"
        "Copyright (C) 2007\n"
        "Alexandru Scvortov\n"
        "scvalex@gmail.com"
    );
    gtk_label_set_justify(GTK_LABEL(copyL), GTK_JUSTIFY_CENTER);
    gtk_box_pack_start_defaults(GTK_BOX(aboutBox), copyL);
    gtk_widget_show(copyL);

    // EndGame
    GtkWidget *egBox; // VButtonBox
    GtkWidget *egMenuBox; // HButtonBox
    GtkWidget *egMainMenuB; // Button
    GtkWidget *egQuitB; // Button

    egBox = gtk_vbutton_box_new();
    gtk_notebook_append_page(GTK_NOTEBOOK(nb), egBox, gtk_label_new("EndGame"));
    gtk_button_box_set_layout(GTK_BUTTON_BOX(egBox), GTK_BUTTONBOX_START);
    gtk_box_set_spacing(GTK_BOX(egBox), 40);
    gtk_container_set_border_width(GTK_CONTAINER(egBox), 40);
    gtk_widget_show(egBox);

    egMenuBox = gtk_vbutton_box_new();
    gtk_box_pack_start(GTK_BOX(egBox), egMenuBox, FALSE, FALSE, 0);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(egMenuBox), GTK_BUTTONBOX_SPREAD);
    gtk_widget_show(egMenuBox);

    egMainMenuB = gtk_button_new_with_label("Main menu");
    gtk_box_pack_start(GTK_BOX(egMenuBox), egMainMenuB, TRUE, FALSE, 0);
    g_signal_connect(GTK_BUTTON(egMainMenuB), "clicked", G_CALLBACK(mainMenuClicked), 0);
    gtk_widget_show(egMainMenuB);

    egQuitB = gtk_button_new_with_label("Quit");
    gtk_box_pack_start(GTK_BOX(egMenuBox), egQuitB, TRUE, FALSE, 0);
    g_signal_connect(GTK_BUTTON(egQuitB), "clicked", G_CALLBACK(quitNow), 0);
    gtk_widget_show(egQuitB);

    egStatusL = gtk_label_new("");
    gtk_box_pack_start_defaults(GTK_BOX(egBox), egStatusL);
    gtk_widget_show(egStatusL);

    egSummaryL = gtk_label_new("");
    gtk_box_pack_start_defaults(GTK_BOX(egBox), egSummaryL);
    gtk_widget_show(egSummaryL);

    srand((unsigned int)egSummaryL);
}

int main(int argc, char *argv[]) {
    gtk_init(&argc, &argv);

    window = gtk_window_new (GTK_WINDOW_TOPLEVEL);
    g_signal_connect (G_OBJECT (window), "destroy",
                      G_CALLBACK (gtk_main_quit),
                      NULL);
    gtk_window_set_title (GTK_WINDOW (window), "C-NC");
    gtk_widget_show(window);

    gtk_widget_set_size_request(GTK_WIDGET(window), 480, 640);
    gtk_container_set_border_width(GTK_CONTAINER(window), 3);

    mainWindow();

    gtk_main();

    return 0;
}
